unit AddNewProject;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, UITypes,
  Dialogs, StdCtrls, DB, ADODB, ExtCtrls;

type
  TFormAddNewProject = class(TForm)
    GroupBox1: TGroupBox;
    ButtonClose: TButton;
    ButtonAdd: TButton;
    LabelName: TLabel;
    LabelTableName: TLabel;
    LabelRegSeed: TLabel;
    EditName: TEdit;
    EditTableName: TEdit;
    EditRegSeed: TEdit;
    ADOConnection: TADOConnection;
    ADOQuery: TADOQuery;
    OpenDialog: TOpenDialog;
    ImageIcon: TImage;
    EditIconFileName: TEdit;
    Label1: TLabel;
    ButtonSelectIcon: TButton;
    DataSource: TDataSource;
    EditDownloadURL: TEdit;
    Label2: TLabel;
    procedure ButtonAddClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure EditNameChange(Sender: TObject);
    procedure ButtonSelectIconClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function SQLAddNewProject(ProjectName: String; TableName: String; RegSeed: String; IconFileName: String; DownloadURL: String): Integer;
    procedure ClearIcon;
  end;

var
  FormAddNewProject: TFormAddNewProject;

implementation

Uses
    Main,
    Settings,
    StrUtils,
    Helper3delite;

{$R *.dfm}

procedure TFormAddNewProject.ButtonAddClick(Sender: TObject);
begin
    Screen.Cursor := crAppStart;
    try
        SQLAddNewProject(EditName.Text, EditTableName.Text, EditRegSeed.Text, EditIconFileName.Text, EditDownloadURL.Text);
        MessageDlg('Successfully created new project "' + EditName.Text + '".', mtConfirmation, [mbOk], 0);
        Screen.Cursor := crDefault;
    except
        on E: Exception do begin
            Screen.Cursor := crDefault;
            MessageDlg('Error: ' + E.Message, mtError, [mbOk], 0);
        end;
    end;
end;

procedure TFormAddNewProject.ButtonCloseClick(Sender: TObject);
begin
    Close;
end;

procedure TFormAddNewProject.ButtonSelectIconClick(Sender: TObject);
begin
    if OpenDialog.Execute then begin
        EditIconFileName.Text := OpenDialog.FileName;
        ImageIcon.Picture.LoadFromFile(EditIconFileName.Text);
    end;
end;

procedure TFormAddNewProject.EditNameChange(Sender: TObject);
begin
    EditTableName.Text := StrConvertToSQLName(EditName.Text);
end;

function TFormAddNewProject.SQLAddNewProject(ProjectName: String; TableName: String; RegSeed: String; IconFileName: String; DownloadURL: String): Integer;
var
    Icon: TIcon;
    IconStream: TStream;
begin

    Result := 0;

    ADOConnection.Close;

    ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;

    ADOQuery.SQL.Clear;

    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');

    //* Insert a new record
    ADOQuery.SQL.Add('INSERT INTO ' + TN_PROJECTS);
    ADOQuery.SQL.Add('(' + CN_PROJECTNAME + ', ' + CN_PROJECT_TABLENAME + ', ' + CN_PROJECT_REG_SEED + ', ' + CN_PROJECT_ICON + ', ' + CN_PROJECT_DOWNLOAD_URL + ') VALUES (:a, :b, :c, :d, :e)');
    //* Specify the values and their data type ftWideString = unicode
    ADOQuery.Parameters.ParamByName('a').DataType := ftWideString;
    ADOQuery.Parameters.ParamByName('a').Value := ProjectName;
    ADOQuery.Parameters.ParamByName('b').DataType := ftString;
    ADOQuery.Parameters.ParamByName('b').Value := TableName;
    ADOQuery.Parameters.ParamByName('c').DataType := ftString;
    ADOQuery.Parameters.ParamByName('c').Value := RegSeed;
    ADOQuery.Parameters.ParamByName('d').DataType := ftVarBytes;
    if FileExists(IconFileName) then begin
        ADOQuery.Parameters.ParamByName('d').LoadFromFile(IconFileName, ftVarBytes);
    end else begin
        Icon := TIcon.Create;
        try
            try
                Icon.Handle := LoadIcon(HInstance, PChar('01DEFAULTPROJECT'));
                IconStream := TMemoryStream.Create;
                try
                    Icon.SaveToStream(IconStream);
                    IconStream.Seek(0, soBeginning);
                    ADOQuery.Parameters.ParamByName('d').LoadFromStream(IconStream, ftVarBytes);
                finally
                    FreeAndNil(IconStream);
                end;
            finally
                FreeAndNil(Icon);
            end;
        except
            //*
        end;
    end;
    ADOQuery.Parameters.ParamByName('e').DataType := ftWideString;
    ADOQuery.Parameters.ParamByName('e').Value := DownloadURL;

    ADOQuery.ExecSql;

    ADOQuery.SQL.Clear;

    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    //* Create a new table
    ADOQuery.SQL.Add('CREATE TABLE "' + TableName + '"');
    ADOQuery.SQL.Add('(');
    //* On the form only 2 columns are specified but we create 4 columns
    //* Add a unique identifier named "Id" with auto-increment
    ADOQuery.SQL.Add('"' + CN_USER_ID + '" INT IDENTITY(100001,1) NOT NULL,');
    //* Add the 2 specified columns as unicode type, max. 50 characters
    ADOQuery.SQL.Add('"' + CN_USER_REGISTRATIONNAME + '" NVARCHAR(255) NOT NULL,');
    ADOQuery.SQL.Add('"' + CN_USER_EMAIL + '" NVARCHAR(255) NOT NULL,');
    ADOQuery.SQL.Add('"' + CN_USER_REG_TIME + '" DATETIME NOT NULL,');
    ADOQuery.SQL.Add('"' + CN_USER_SHAREITORDEREMAIL + '" NTEXT,');
    //* V4
    ADOQuery.SQL.Add('"' + CN_USER_SALUTATION + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_TITLE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_LASTNAME + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_FIRSTNAME + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_COMPANY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_CONTACTNAME + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_ADDRESS + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_CITY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_FULLCITY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_COUNTRY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_POSTALCODE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_STATEPROVINCE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_TELEPHONE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_MOBILEPHONE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_FAX + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_HOMEPAGE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_NEWSLETTER + '" BIT,');
    ADOQuery.SQL.Add('"' + CN_USER_VATID + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_NOOFLICENSES + '" INT,');
    ADOQuery.SQL.Add('"' + CN_USER_LANGUAGE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_SHAREITREFNO + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_RESELLER + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_PROMOTIONNAME + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_NETSALESCURRENCY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_NETSALES + '" MONEY,');
    ADOQuery.SQL.Add('"' + CN_USER_NETDISCOUNTCURRENCY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_NETDISCOUNT + '" MONEY,');
    ADOQuery.SQL.Add('"' + CN_USER_COLLECTEDVATCURRENCY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_COLLECTEDVAT + '" MONEY,');
    ADOQuery.SQL.Add('"' + CN_USER_SHIPPINGCURRENCY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_SHIPPING + '" MONEY,');
    ADOQuery.SQL.Add('"' + CN_USER_VATSHIPPINGCURRENCY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_VATSHIPPING + '" MONEY,');
    ADOQuery.SQL.Add('"' + CN_USER_TOTALCURRENCY + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_TOTAL + '" MONEY,');
    ADOQuery.SQL.Add('"' + CN_USER_PAYMENT + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_DELIVERYMODE + '" NVARCHAR(255),');
    ADOQuery.SQL.Add('"' + CN_USER_COMMENTS + '" NTEXT,');
    //* Id column as primary key
    ADOQuery.SQL.Add('PRIMARY KEY (' + CN_USER_ID + ')');
    ADOQuery.SQL.Add(')');

    ADOQuery.ExecSql;

    //* Read back new ID
    ADOQuery.Close;
    ADOQuery.SQL.Clear;

    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    //* Get all data
    ADOQuery.SQL.Add('SELECT * FROM ' + TN_PROJECTS + ' WHERE ' + CN_PROJECTNAME + ' = ''' + ProjectName + '''');
    ADOQuery.Open;

    DataSource.DataSet.First;
    while NOT DataSource.DataSet.Eof do begin
        Result := DataSource.DataSet.FieldByName(CN_PROJECT_ID).AsInteger;
        DataSource.DataSet.Next;
    end;

    ADOConnection.Close;

end;

procedure TFormAddNewProject.ClearIcon;
var
    Icon: TIcon;
begin
    ImageIcon.Picture.Assign(nil);
    EditIconFileName.Text := 'Select an icon file...';
    Icon := TIcon.Create;
    try
        try
            Icon.Handle := LoadIcon(HInstance, PChar('01DEFAULTPROJECT'));
            ImageIcon.Picture.Assign(Icon);
        finally
            FreeAndNil(Icon);
        end;
    except
        //*
    end;
end;

end.
