unit MailProcessing;

interface

Uses
    SysUtils,
    Classes,
    Dialogs,
    UITypes,
    IDPOP3,
    IDSSL,
    IdSSLOpenSSL,
    IdExplicitTLSClientServerBase,
    IdMessage,
    IdIMAP4,
    IdText,
    IdComponent,
    IdTCPConnection,
    IdTCPClient,
    IdMessageClient,
    IdSMTPBase,
    IdSMTP,
    IdBaseComponent,
    Generics.Collections,
    Forms,
    Controls;

    procedure CheckAndProcessIncomingMail;

    procedure FillProjectsList;

type
    TProjectItem = record
        Name: String;
        ID: String;
        TableName: String;
        RegSeed: String;
        DownloadURL: String;
    end;

var
    ProjectsList: TList<TProjectItem>;

implementation

Uses
    StrUtils,
    ComCtrls,
    Windows,
    QueryEmails,
    Main,
    Settings,
    UserDetails,
    ProjectManager,
    CalcReg,
    Helper3delite,
    System.Character,
    DateUtils,
    IdStrings,
    System.NetEncoding
    //IdAttachmentFile, IdSASL, IdSASLCollection, IdSASLAnonymous, IdSASLDigest, IdSASLExternal, IdSASLLogin, IdSASLOTP, IdSASLPlain, IdSASLSKey, IdSASLUserPass, IdSASL_CRAM_MD5, IdSASL_CRAM_SHA1
    ;

procedure LogStrings(FileName, Text: String);
var
    MonthlyLogFileName: String;
    CurrentDate: TDateTime;
    MonthStr: String;
begin
    Helper3delite.LogStrings(FormSettings.EditLogFileName.Text, Text);
    if Length(IncludeTrailingPathDelimiter(FormSettings.EditMonthlyLogFolder.Text)) > 1 then begin
        CurrentDate := Now;
        MonthStr := IntToStr(MonthOf(CurrentDate));
        if Length(MonthStr) = 1 then begin
            MonthStr := '0' + MonthStr;
        end;
        MonthlyLogFileName := IncludeTrailingPathDelimiter(FormSettings.EditMonthlyLogFolder.Text) + IntToStr(YearOf(CurrentDate)) + '.' + MonthStr + '.log';
        ForceDirectories(FormSettings.EditMonthlyLogFolder.Text);
        Helper3delite.LogStrings(MonthlyLogFileName, Text);
    end;
end;

procedure FreeEmails(Emails: TList<TIdMessage>); overload;
var
    i: Integer;
begin
    for i := 0 to Emails.Count - 1 do begin
        if Assigned(Emails[i]) then begin
            Emails[i].Free;
            Emails[i] := nil;
        end;
    end;
    Emails.Clear;
end;

function ConnectPOP3(POP3: TIDPOP3; SSL: TIdSSLIOHandlerSocketOpenSSL; AccountItemData: TAccountItemData): Boolean; overload;
begin
    Result := False;
    try
        //* Params
        POP3.ConnectTimeout := 10000;
        POP3.ReadTimeout := 10000;
        //* Server & login
        POP3.Host := AccountItemData.HostName;
        POP3.Port := AccountItemData.Port;
        POP3.Username := AccountItemData.LoginName;
        POP3.Password := AccountItemData.Password;
        //* SSL
        if AccountItemData.Security = asSSLTLS then begin
            SSL.Host := POP3.Host;
            SSL.Port := POP3.Port;
            SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
            SSL.SSLOptions.Method := AccountItemData.SSLMethod; //sslvSSLv3; // sslvTLSv1_2
            POP3.IOHandler := SSL;
            POP3.UseTLS := utUseImplicitTLS;
        end;
        // Try to connect
        POP3.Connect;
        Result := True;
    except
        //* ???
    end;
end;

function ConnectIMAP(IMAPClient: TIdIMAP4; OpenSSLHandler: TIdSSLIOHandlerSocketOpenSSL;AccountItemData: TAccountItemData): Boolean; overload;
begin
    Result := False;
    try
        if AccountItemData.Security = asSSLTLS then begin
            OpenSSLHandler.SSLOptions.Method := sslvSSLv3;
            IMAPClient.IOHandler := OpenSSLHandler;
            IMAPClient.UseTLS := utUseImplicitTLS;
        end;
        IMAPClient.Host := AccountItemData.HostName;
        IMAPClient.Port := AccountItemData.Port;
        IMAPClient.Username := AccountItemData.LoginName;
        IMAPClient.Password := AccountItemData.Password;
        IMAPClient.Connect;
        Result := True;
    except
        //*
    end;
end;

function GetEmailsPOP3(POP3: TIDPOP3; Emails: TList<TIdMessage>; OnlyHeaders: Boolean = False): Boolean; overload;
var
    MsgCnt: Integer;
    i: Integer;
    Msg: TIdMessage;
begin
    (*
    Msg := TIdMessage.Create;
    Msg.LoadFromFile('E:\Test.eml');
    Emails.Add(Msg);
    Result := True;
    *)
    Result := False;
    try
        //* Are there new messages?
        MsgCnt := POP3.CheckMessages;
        //* Parse the messages
        for i := 1 to MsgCnt do begin
            Msg := TIdMessage.Create;
            if OnlyHeaders then begin
                POP3.RetrieveHeader(i, Msg);
            end else begin
                POP3.Retrieve(i, Msg);
            end;
            //* Do something with it
            Emails.Add(Msg);
        end;
        if Emails.Count > 0 then begin
            Result := True;
        end;
    except
        //* Error with connection
    end;
    //*)
end;

function GetEmailsIMAP(IMAPClient: TIdIMAP4; Emails: TList<TIdMessage>; OnlyHeaders: Boolean = False): Boolean; overload;
var
    MsgIndex: Integer;
    MsgObject: TIdMessage;
    //PartIndex: Integer;
begin
    Result := False;
    try
        if IMAPClient.SelectMailBox('INBOX') then begin
            for MsgIndex := 1 to IMAPClient.MailBox.TotalMsgs do begin
                MsgObject := TIdMessage.Create(nil);
                if OnlyHeaders then begin
                    IMAPClient.RetrieveHeader(MsgIndex, MsgObject);
                end else begin
                    IMAPClient.Retrieve(MsgIndex, MsgObject);
                end;
                //* Do something with it
                Emails.Add(MsgObject);
            end;
            if Emails.Count > 0 then begin
                Result := True;
            end;
        end;
    except
        //*
    end;
end;

function CloneEmail(FromEmail: TIdMessage; ToEmail: TIdMessage): Boolean;
var
    EmailStream: TMemoryStream;
begin
    EmailStream := TMemoryStream.Create;
    try
        FromEmail.SaveToStream(EmailStream);
        EmailStream.Seek(0, soBeginning);
        ToEmail.LoadFromStream(EmailStream);
        Result := True;
    finally
        FreeAndNil(EmailStream);
    end;
end;

function SendEmailsList(EmailAccount: TAccountItemData; SendEmails: TList<TIdMessage>): Boolean;
var
    i: Integer;
    IdSMTP: TIdSMTP;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
    CurrentCounter: Integer;
begin
    Result := False;
    if SendEmails.Count = 0 then begin
        Exit;
    end;
    SSL := nil;
    IdSMTP := TIdSMTP.Create;
    if EmailAccount.Security = asSSLTLS then begin
        SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
    end;
    try
        //setup SMTP
        IdSMTP.Host := EmailAccount.HostName;
        IdSMTP.Port := EmailAccount.Port;
        IdSMTP.Username := EmailAccount.LoginName;
        IdSMTP.Password := EmailAccount.Password;
        if EmailAccount.Security = asSSLTLS then begin
            SSL.Host := IdSMTP.Host;
            SSL.Port := IdSMTP.Port;
            SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
            SSL.SSLOptions.Method := EmailAccount.SSLMethod; //sslvSSLv3; {good} // sslvTLSv1_2 for 3delite.hu
            IdSMTP.IOHandler := SSL;
            IdSMTP.UseTLS := utUseImplicitTLS;
        end;
        try
            CurrentCounter := - 1;
            try
                IdSMTP.Connect;
                for i := 0 to SendEmails.Count - 1 do begin
                    CurrentCounter := i;
                    IdSMTP.Send(SendEmails[i]);
                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' E-mail sent: ''' + SendEmails[i].Recipients[0].Name + ''' <' + SendEmails[i].Recipients[0].Address + '> - ' + SendEmails[i].Subject);
                end;
                Result := True;
            except on E:Exception do begin
                    Result := False;
                    if CurrentCounter > - 1 then begin
                        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' E-mail send EXCEPTION: ''' + SendEmails[CurrentCounter].Recipients[0].Name + ''' <' + SendEmails[CurrentCounter].Recipients[0].Address + '> - ' + SendEmails[CurrentCounter].Subject + #13#10 + E.Message);
                    end else begin
                        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' E-mail send EXCEPTION: ' + E.Message);
                    end;
                    MessageDlg('ERROR: ' + E.Message, mtWarning, [mbCancel], 0);
                end;
            end;
        finally
            if IdSMTP.Connected then begin
                IdSMTP.Disconnect;
            end;
        end;
    finally
        FreeAndNil(IdSMTP);
        if Assigned(SSL) then begin
            FreeAndNil(SSL);
        end;
    end;
end;

function AddEmailToDataBase(Email: TIdMessage; ProjectItem: TProjectItem; SendEmails: TList<TIdMessage>): Boolean;
var
    NewRegistrantID: Integer;
    UserName, UserEmail: String;
    SubVersionSeed: Double;
    StrBody: String;
    EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError: String;
    RegNumText, VerCodeText: String;
    SubVersionName: String;
    SendEmail: TIdMessage;
    BodyHTML: TStringList;
    i: Integer;
begin
    Result := False;

    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' AddEmailToDataBase: ' + Email.Subject);

    try
        FormRegistrantDetails.Clear;
        FormRegistrantDetails.ProjectName := ProjectItem.Name;

        (*
        if {SameText(Email.ContentType, 'text/html')
        AND} SameText(Email.ContentTransferEncoding, 'base64')
        then begin
            BodyHTML := TStringList.Create;
            try
                BodyHTML.Text := TNetEncoding.Base64.Decode(Email.Body.Text);
                FormRegistrantDetails.ProcessShareItEMail(BodyHTML);
            finally
                FreeAndNil(BodyHTML);
            end;
        end else begin
        *)

        if Email.ContentType = 'multipart/alternative' then begin
            for i := 0 to Email.MessageParts.Count - 1 do begin
                if Email.MessageParts[i].ContentType = 'text/plain' then begin
                    if SameText(Email.ContentTransferEncoding, 'base64') then begin
                        BodyHTML := TStringList.Create;
                        try
                            BodyHTML.Text := TNetEncoding.Base64.Decode(TIdText(Email.MessageParts.Items[i]).Body.Text);
                            FormRegistrantDetails.ProcessShareItEMail(BodyHTML);
                        finally
                            FreeAndNil(BodyHTML);
                        end;
                    end else begin
                        FormRegistrantDetails.ProcessShareItEMail(TIdText(Email.MessageParts.Items[i]).Body);
                        Break;
                    end;
                end;
            end;
        end else begin
            FormRegistrantDetails.ProcessShareItEMail(Email.Body);
        end;

        if FormRegistrantDetails.OrderAlreadyInDataBase(ProjectItem.TableName, FormRegistrantDetails.EditShareItRefNo.Text) then begin
            SendEmail := TIdMessage.Create;
            CloneEmail(Email, SendEmail);
            SendEmail.From.Address := FormSettings.EditSenderEmail.Text;
            SendEmail.Recipients.EMailAddresses := FormSettings.EditForwardAddress.Text;
            SendEmail.Subject := 'DUPLICATE (ref. no. already in db.): ' + Email.Subject;
            SendEmails.Add(SendEmail);
            Exit;
        end;

        UserName := FormRegistrantDetails.EditCustomerRegistrationName.Text;
        UserEmail := FormRegistrantDetails.EditCustomerEMail.Text;

        NewRegistrantID := FormTool.SQLAddNewProjectRegistration(ProjectItem.TableName, UserName, UserEmail, Now, Email.Body.Text);

        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' NewRegistrantID: ' + IntToStr(NewRegistrantID));

        FormRegistrantDetails.EditCustomerID.Text := IntToStr(NewRegistrantID);
        FormRegistrantDetails.SQLUpdateRegistration(ProjectItem.TableName);

        FormTool.GetLastSubVersion(ProjectItem.ID, SubVersionSeed, SubVersionName);

        RegNumText := IntToStr(NewRegistrantID) + IntToStr(CalcRegNum(UserName, UserEmail, StrToFloat(ProjectItem.RegSeed)));
        VerCodeText := IntToStr(CalcVerCode(StrToInt64(RegNumText), SubVersionSeed));

        FormProjectManager.SQLLoadProjectEMailTemplates(StrToInt(ProjectItem.ID), EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError);

        if EMailTemplateRegistration = '' then begin
            FormTool.SQLLoadDefaultEMailTemplates(EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError);
        end;

        //* No template found (should not happen)
        if EMailTemplateRegistration = '' then begin

            StrBody := 'Name: ' + UserName + #13#10
                + 'E-mail: ' + UserEmail + #13#10
                + 'Your Registration number: ' + RegNumText + #13#10
                + 'Your Version code: ' + VerCodeText + #13#10
                + ProjectItem.Name + ' Version: ' + SubVersionName;

            //StrBody := StringReplace(StrBody, '&&', '&', [rfReplaceAll]);
            //StrBody := StringReplace(StrBody, '&', '%26', [rfReplaceAll]);

        //* Fill in tamplete varialbes
        end else begin
            StrBody := EMailTemplateRegistration;
            StrBody := StringReplace(StrBody, '%PROGRAMNAME%', ProjectItem.Name, [rfReplaceAll]);
            StrBody := StringReplace(StrBody, '%PROGRAMVERSION%', SubVersionName, [rfReplaceAll]);
            StrBody := StringReplace(StrBody, '%REGISTRATIONNAME%', UserName, [rfReplaceAll]);
            StrBody := StringReplace(StrBody, '%REGISTRATIONEMAIL%', UserEmail, [rfReplaceAll]);
            StrBody := StringReplace(StrBody, '%REGISTRATIONNUMBER%', RegNumText, [rfReplaceAll]);
            StrBody := StringReplace(StrBody, '%REGISTRATIONVERSIONCODE%', VerCodeText, [rfReplaceAll]);
            StrBody := StringReplace(StrBody, '%PROGRAMDOWNLOADURL%', ProjectItem.DownloadURL, [rfReplaceAll]);
        end;

        //StrBody := URLEncode(StrBody , False); *

        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Sending registration data: ' + IntToStr(NewRegistrantID));

        SendEmail := TIdMessage.Create;
        SendEmail.ContentType := 'text/plain';
        SendEmail.CharSet := 'UTF-8';
        SendEmail.From.Address := FormSettings.EditSenderEmail.Text;
        SendEmail.ReplyTo.Add.Address := FormSettings.EditSenderEmail.Text;
        SendEmail.Recipients.EMailAddresses := UserEmail;
        SendEmail.Subject := ProjectItem.Name + ' - Your Registration Number';
        SendEmail.Body.Text := StrBody;
        SendEmails.Add(SendEmail);
        Result := True;

        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Sending registration data finished: ' + IntToStr(NewRegistrantID) {+ #13#10});

    except
        On E: Exception do begin
            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' EXCEPTION: AddEmailToDataBase: ' + E.Message);
        end;
    end;
end;

function CheckEmailListToAddToDatabase(IncomingEmail: TIdMessage; SendEmails: TList<TIdMessage>): Boolean;
Const
    EMAIL_ORDER_ID = 'Order No.';
var
    k: Integer;
    SendEmail: TIdMessage;
    EmailSubject: String;
    SuccesfullyAddedToDataBase: Boolean;
begin
    Result := False;
    SuccesfullyAddedToDataBase := False;
    if FormSettings.CheckBoxLogDebug.Checked then begin
        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' CheckEmailListToAddToDatabase: =================================================');
    end;

    //SendEmail := nil;

    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.Subject: ' + IncomingEmail.Subject);

    if Copy(IncomingEmail.Subject, 1, Length(EMAIL_ORDER_ID)) <> EMAIL_ORDER_ID then begin
        Exit;
    end;

    EmailSubject := StringReplace(IncomingEmail.Subject, '&amp;', '&', [rfReplaceAll]);

    for k := 0 to ProjectsList.Count - 1 do begin

        if FormSettings.CheckBoxLogDebug.Checked then begin
            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' EmailSubject: ' + EmailSubject + ' <> ' + ProjectsList[k].Name);
        end;

        if ContainsText(EmailSubject, ProjectsList[k].Name) then begin
            SuccesfullyAddedToDataBase := AddEmailToDataBase(IncomingEmail, ProjectsList[k], SendEmails);
            Break;
        end else begin
            if FormSettings.CheckBoxLogDebug.Checked then begin
                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' EmailSubject: ' + EmailSubject + ' <> NOT project ' + ProjectsList[k].Name);
            end;
        end;
    end;

    //*------------------------------ All bundle
    if FormSettings.EditAllBundleProductName.Text <> '' then begin
        if ContainsText(EmailSubject, FormSettings.EditAllBundleProductName.Text) then begin
            for k := 0 to ProjectsList.Count - 1 do begin
                SuccesfullyAddedToDataBase := AddEmailToDataBase(IncomingEmail, ProjectsList[k], SendEmails);
            end;
        end;
    end;

    if NOT SuccesfullyAddedToDataBase then begin
        SendEmail := TIdMessage.Create;
        CloneEmail(IncomingEmail, SendEmail);
        SendEmail.From.Address := FormSettings.EditSenderEmail.Text;
        SendEmail.Recipients.EMailAddresses := FormSettings.EditForwardAddress.Text;
        SendEmail.Subject := 'ATTENTION NEEDED (not processed): ' + IncomingEmail.Subject;
        SendEmails.Add(SendEmail);

        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' ATTENTION NEEDED (not processed) Email.Subject: ' + IncomingEmail.Subject);
        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.Body: ' + IncomingEmail.Body.Text);

    end;
    Result := SuccesfullyAddedToDataBase;
end;

function ValidVersionCodeRequestEmail(Email: TIdMessage; var ProjectName: String): Boolean;
var
    AppendPos: Integer;
begin
    Result := False;
    ProjectName := '';
    AppendPos := Pos(' - Request Version Code', Email.Subject);
    if AppendPos > 0 then begin
        ProjectName := Copy(Email.Subject, 1, AppendPos - 1);
        Result := True;
    end;
    {$IFDEF DEBUG}
    //LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' ValidVersionCodeRequestEmail: ' + Email.Subject + ' -> ' + ProjectName);
    {$ENDIF}
end;

function ExtractUserIDAndVersion(ProjectName: String; Text: TStrings; var UserName, UserID: String; var Version: String): Boolean;
var
    i, k: Integer;
    StartPos: Integer;
    VersionCodeChar: Char;
begin
    //Result := False;
    UserID := '';
    Version := '';
    for i := 0 to Text.Count - 1 do begin
        StartPos := Pos('Registration Number: ', Text[i]);
        if StartPos > 0 then begin
            UserID := Copy(Text[i], StartPos + Length('Registration Number: '), 6);
        end;
        StartPos := Pos(ProjectName, Text[i]);
        if StartPos > 0 then begin
            Version := '';
            for k := StartPos + Length(ProjectName) + 1 to Length(Text[i]) do begin
                VersionCodeChar := Text[i][k];
                if VersionCodeChar.IsDigit
                OR (VersionCodeChar = '.')
                then begin
                    Version := Version + VersionCodeChar;
                end else begin
                    Break;
                end;
            end;
        end;

        StartPos := Pos('Name: ', Text[i]);
        if StartPos > 0 then begin
            UserName := Copy(Text[i], StartPos + Length('Name: '), Length(Text[i]));
            if Pos('E-mail', UserName) > 0 then begin
                UserName := Copy(UserName, 1, Pos('E-mail', UserName) - 1);
            end;
            if Pos('<', UserName) > 0 then begin
                UserName := Copy(UserName, 1, Pos('<', UserName) - 1);
            end;
            UserName := Trim(UserName);
        end;

        if (UserID <> '')
        AND (Version <> '')
        AND (UserName <> '')
        then begin
            Break;
        end;
    end;
    Result := (UserID <> '') AND (Version <> '') AND (UserName <> '');
    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' ExtractUserIDAndVersion: UserName = ' + UserName + ', UserID = ' + UserID + ', Version = ' + Version);
end;

function ProcessVersionCodeRequestEmail(Email: TIdMessage; ProjectName: String; OutgoingEmailAccount: TAccountItemData): Boolean;
var
    i, k: Integer;
    ProjectTableName: String;
    UserName: String;
    UserID: String;
    Version: String;
    RegistrantDetails: TRegistrantDetails;
    RegNumText, VerCodeText: String;
    SubVersionSeed: Double;
    EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError: String;
    StrBody: String;
    SendEmail: TIdMessage;
    SendEmails: TList<TIdMessage>;
    MessageBody: TStringList;
    MessageStream: TMemoryStream;
begin
    Result := False;
    SendEmails := TList<TIdMessage>.Create;
    MessageBody := TStringList.Create;
    try
        for k := 0 to ProjectsList.Count - 1 do begin
            if SameText(ProjectName, ProjectsList[k].Name) then begin
                ProjectTableName := ProjectsList[k].TableName;
                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' ProcessVersionCodeRequestEmail: ProjectTableName = ' + ProjectTableName);
                try
                    if SameText(Email.ContentType, 'text/plain') then begin
                        MessageBody.Text := Email.Body.Text;
                        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.ContentType = text/plain');
                    end else if Email.ContentType = 'multipart/alternative' then begin
                        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.ContentType = multipart/alternative');
                        for i := 0 to Email.MessageParts.Count - 1 do begin
                            if Email.MessageParts[i].ContentType = 'text/plain' then begin
                                if SameText(Email.ContentTransferEncoding, 'base64') then begin
                                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.ContentTransferEncoding = base64');
                                    MessageBody.Text := MessageBody.Text + #13#10 + TNetEncoding.Base64.Decode(TIdText(Email.MessageParts.Items[i]).Body.Text);
                                    //Break;
                                end else begin
                                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.ContentTransferEncoding <> base64');
                                    MessageBody.Text := MessageBody.Text + #13#10 + TIdText(Email.MessageParts.Items[i]).Body.Text;
                                    //Break;
                                end;
                            end else begin
                                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Email.MessageParts[i].ContentType <> text/plain');
                                MessageBody.Text := MessageBody.Text + #13#10 + StrHTMLDecode(TIdText(Email.MessageParts.Items[i]).Body.Text);
                                //Break;
                                //MessageStream := TMemoryStream.Create;
                                //try
                                    //Email.SaveToStream(MessageStream);
                                    //MessageStream.Seek(0, soBeginning);
                                    //MessageBody.LoadFromStream(MessageStream);
                                    //MessageBody.Text := StrHTMLDecode(MessageBody.Text);
                                //finally
                                //    FreeAndNil(MessageStream);
                                //end;
                            end;
                        end;
                    end else begin
                        MessageStream := TMemoryStream.Create;
                        try
                            Email.SaveToStream(MessageStream);
                            MessageStream.Seek(0, soBeginning);
                            MessageBody.LoadFromStream(MessageStream);
                            //MessageBody.Text := StrHTMLDecode(MessageBody.Text);
                        finally
                            FreeAndNil(MessageStream);
                        end;
                    end;
                    MessageBody.Text := StrHTMLDecode(MessageBody.Text);
                    //MessageBody.SaveToFile('E:\RegTool ' + StrConvertToFileName(DateTimeToStr(Now)) + ' MessageBody.txt');
                except
                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' ProcessVersionCodeRequestEmail: EXCEPTION: MessageBody');
                end;

                if ExtractUserIDAndVersion(ProjectName, MessageBody, UserName, UserID, Version) then begin
                    if FormRegistrantDetails.SQLLoadRegistration(ProjectTableName, UserID, RegistrantDetails) > 0 then begin
                        if RegistrantDetails.RegistrationName <> UserName then begin
                            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' RegistrationName <> UserName: ' + RegistrantDetails.RegistrationName + ' <> ' + UserName);
                            (*
                            //* Fill in tamplete varialbes
                            StrBody := 'Sorry, the specified user could not be found in the database.' + #13#10;
                            StrBody := StrBody + #13#10;
                            StrBody := StrBody + 'Please contact: ' + FormSettings.EditSupportAddress.Text + #13#10;
                            //* Send the notice e-mail
                            SendEmail := TIdMessage.Create;
                            SendEmail.ContentType := 'text/plain';
                            SendEmail.CharSet := 'UTF-8';
                            SendEmail.From.Address := FormSettings.EditSenderEmail.Text;
                            SendEmail.ReplyTo.Add.Address := FormSettings.EditSenderEmail.Text;
                            with SendEmail.Recipients.Add do begin
                                Name := Email.From.Name;
                                Address := Email.From.Address;
                            end;
                            SendEmail.Subject := 'Re: ' + Email.Subject;
                            SendEmail.Body.Text := StrBody;
                            SendEmails.Add(SendEmail);
                            Continue;
                            *)
                        end;
                        if FormTool.GetSubVersionSeed(ProjectsList[k].ID, Version, SubVersionSeed) then begin
                            RegNumText := UserID + IntToStr(CalcRegNum(RegistrantDetails.RegistrationName, RegistrantDetails.Email, StrToFloat(ProjectsList[k].RegSeed)));
                            VerCodeText := IntToStr(CalcVerCode(StrToInt64(RegNumText), SubVersionSeed));
                            FormProjectManager.SQLLoadProjectEMailTemplates(StrToInt(ProjectsList[k].ID), EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError);
                            if EMailTemplateRegistration = '' then begin
                                FormTool.SQLLoadDefaultEMailTemplates(EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError);
                            end;
                            //* No template found (should not happen)
                            if EMailTemplateVersionCode = '' then begin
                                StrBody := 'Name: ' + RegistrantDetails.RegistrationName + #13#10
                                    + 'E-mail: ' + RegistrantDetails.Email + #13#10
                                    + 'Your Version code: ' + VerCodeText + #13#10
                                    + ProjectsList[k].Name + ' Version: ' + Version;
                            //* Fill in tamplete varialbes
                            end else begin
                                StrBody := EMailTemplateVersionCode;
                                StrBody := StringReplace(StrBody, '%PROGRAMNAME%', ProjectsList[k].Name, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%PROGRAMVERSION%', Version, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONNAME%', RegistrantDetails.RegistrationName, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONEMAIL%', RegistrantDetails.Email, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONNUMBER%', RegNumText, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONVERSIONCODE%', VerCodeText, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%PROGRAMDOWNLOADURL%', ProjectsList[k].DownloadURL, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%SUPPORTEMAIL%', FormSettings.EditSupportAddress.Text, [rfReplaceAll]);
                            end;
                            //* Send the new version code
                            SendEmail := TIdMessage.Create;
                            SendEmail.ContentType := 'text/plain';
                            SendEmail.CharSet := 'UTF-8';
                            SendEmail.From.Address := FormSettings.EditSenderEmail.Text;
                            SendEmail.ReplyTo.Add.Address := FormSettings.EditSenderEmail.Text;
                            with SendEmail.Recipients.Add do begin
                                Name := RegistrantDetails.RegistrationName;
                                Address := RegistrantDetails.Email;
                            end;
                            SendEmail.Subject := ProjectsList[k].Name + ' - Your Version Code';
                            SendEmail.Body.Text := StrBody;
                            SendEmails.Add(SendEmail);
                            //* Forward the processed request e-mail to ourselves
                            {
                            SendEmail := TIdMessage.Create;
                            CloneEmail(Email, SendEmail);
                            SendEmail.Subject := 'OK: ' + SendEmail.Subject;
                            SendEmail.Recipients.EMailAddresses := FormSettings.EditForwardAddress.Text;
                            SendEmails.Add(SendEmail);
                            }
                            //* Successfully generated the new version code e-mail.
                            Result := True;
                            //* Request e-mail address is not the 1 in database: send a notice
                            if NOT ContainsText(Email.From.Address, RegistrantDetails.Email) then begin
                                //* No template found (should not happen)
                                if EMailTemplateVersionCodeError = '' then begin
                                    EMailTemplateVersionCodeError := FormTool.MemoDefaultEMailTemplateVersionCodeError.Text;
                                end;
                                //* Fill in tamplete varialbes
                                StrBody := EMailTemplateVersionCodeError;
                                StrBody := StringReplace(StrBody, '%PROGRAMNAME%', ProjectsList[k].Name, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%PROGRAMVERSION%', Version, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONNAME%', RegistrantDetails.RegistrationName, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONEMAIL%', Copy(RegistrantDetails.Email, 1, 1) + '***' + Copy(RegistrantDetails.Email, Pos('@', RegistrantDetails.Email), Length(RegistrantDetails.Email)), [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONNUMBER%', RegNumText, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%REGISTRATIONVERSIONCODE%', VerCodeText, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%PROGRAMDOWNLOADURL%', ProjectsList[k].DownloadURL, [rfReplaceAll]);
                                StrBody := StringReplace(StrBody, '%SUPPORTEMAIL%', FormSettings.EditSupportAddress.Text, [rfReplaceAll]);
                                //* Send the notice e-mail
                                SendEmail := TIdMessage.Create;
                                SendEmail.ContentType := 'text/plain';
                                SendEmail.CharSet := 'UTF-8';
                                SendEmail.From.Address := FormSettings.EditSenderEmail.Text;
                                SendEmail.ReplyTo.Add.Address := FormSettings.EditSenderEmail.Text;
                                with SendEmail.Recipients.Add do begin
                                    Name := Email.From.Name;
                                    Address := Email.From.Address;
                                end;
                                SendEmail.Subject := 'Re: ' + Email.Subject;
                                SendEmail.Body.Text := StrBody;
                                SendEmails.Add(SendEmail);
                                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' SendEmails.Added: Name: ' + Email.From.Name + ' Address: ' + Email.From.Address);
                            end;
                        end;
                    end;
                end;
                //LogStrings(FormSettings.EditLogFileName.Text, '');
            end;
        end;
        //* There was an error while processing this version code request e-mail
        if NOT Result then begin
            //* Forward the un-processed request e-mail
            SendEmail := TIdMessage.Create;
            CloneEmail(Email, SendEmail);
            SendEmail.Subject := 'ATTENTION, ERROR PROCESSING: ' + SendEmail.Subject;
            SendEmail.Recipients.EMailAddresses := FormSettings.EditForwardAddress.Text;
            SendEmails.Add(SendEmail);
            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' SendEmails.Added, ERROR PROCESSING: Name: ' + Email.From.Name + ' Address: ' + Email.From.Address);
            //Result := True;
        end;
        //* Send all queued e-mails
        Result := SendEmailsList(OutgoingEmailAccount, SendEmails);
    finally
        FreeEmails(SendEmails);
        FreeAndNil(SendEmails);
        FreeAndNil(MessageBody);
    end;
end;

procedure CheckAndProcessIncomingMail;
var
    IncomingEmails: TList<TIdMessage>;
    SendEmails: TList<TIdMessage>;
    POP3: TIDPOP3;
    IMAP: TIdIMAP4;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
    IncomingEmailAccount: TAccountItemData;
    IncomingEmailAccountVerCode: TAccountItemData;
    OutgoingEmailAccount: TAccountItemData;
    ValidVersionCodeRequestEmailReceived: Boolean;
    i, k: Integer;
    ProjectName: String;
    ProcessedOrderEmail: Boolean;
    Msg: TIdMessage;
    ForwardEmail: TIdMessage;
    IdSMTP: TIdSMTP;
    SMTPSSL: TIdSSLIOHandlerSocketOpenSSL;
    //ErrorSendingRegistrationEMail: Boolean;
    Success: Boolean;
  {
  FIdSASLCRAMSHA1: TIdSASLCRAMSHA1;
  FIdSASLCRAMMD5: TIdSASLCRAMMD5;
  FIdSASLDigest: TIdSASLDigest;
  FIdSASLSKey: TIdSASLSKey;
  FIdSASLOTP: TIdSASLOTP;
  FIdSASLAnonymous: TIdSASLAnonymous;
  FIdSASLExternal: TIdSASLExternal;
  FIdSASLLogin: TIdSASLLogin;
  FIdSASLPlain: TIdSASLPlain;
  }
begin

    (*
    Msg := TIdMessage.Create(nil);
    Msg.LoadFromFile('E:\RegTool 2020. 11. 03. 13_16_45 MessageBody.txt');
    ProcessVersionCodeRequestEmail(Msg, 'MKV Tag Editor', nil);
    Exit;
    *)

    Screen.Cursor := crAppStart;
    POP3 := nil;
    IMAP := nil;
    SSL := nil;
    ProcessedOrderEmail := False;
    //ErrorSendingRegistrationEMail := False;
    IncomingEmailAccount := FormSettings.IncomingEmailAccount;
    IncomingEmailAccountVerCode := FormSettings.IncomingEmailAccountVersionCodes;
    OutgoingEmailAccount := FormSettings.OutgoingEmailAccount;

    try
        if (IncomingEmailAccountVerCode.HostName <> '')
        AND (IncomingEmailAccountVerCode.LoginName <> '')
        AND (IncomingEmailAccountVerCode.Password <> '')
        then begin
            if FormSettings.CheckBoxLogDebug.Checked then begin
                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Parsing incoming e-mails VERCODE...');
            end;
            IncomingEmails := TList<TIdMessage>.Create;
            try
                case IncomingEmailAccountVerCode.ServerType of
                    asPOP3: POP3 := TIDPOP3.Create;
                    asIMAP: IMAP := TIdIMAP4.Create;
                end;
                if IncomingEmailAccountVerCode.Security = asSSLTLS then begin
                    SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
                end;
                case IncomingEmailAccountVerCode.ServerType of
                    asPOP3: begin
                        if ConnectPOP3(POP3, SSL, IncomingEmailAccountVerCode) then begin
                            GetEmailsPOP3(POP3, IncomingEmails, True); //* BUG: body is empty if first querying without headers...
                        end;
                    end;
                    asIMAP: begin
                        if ConnectIMAP(IMAP, SSL, IncomingEmailAccountVerCode) then begin
                            GetEmailsIMAP(IMAP, IncomingEmails, True);
                        end;
                    end;
                end;
                ValidVersionCodeRequestEmailReceived := False;
                for i := 0 to IncomingEmails.Count - 1 do begin
                    if ValidVersionCodeRequestEmail(IncomingEmails[i], ProjectName) then begin
                        ValidVersionCodeRequestEmailReceived := True;
                        //* BUG: body is empty if first querying without headers...
                        IncomingEmails[i].Free;
                        IncomingEmails[i] := TIdMessage.Create;
                        case IncomingEmailAccountVerCode.ServerType of
                            asPOP3: begin
                                POP3.Retrieve(i + 1, IncomingEmails[i]);
                            end;
                            asIMAP: begin
                                IMAP.Retrieve(i + 1, IncomingEmails[i]);
                            end;
                        end;
                        //}
                        if ProcessVersionCodeRequestEmail(IncomingEmails[i], ProjectName, OutgoingEmailAccount) then begin
                            case IncomingEmailAccountVerCode.ServerType of
                                asPOP3: POP3.Delete(i + 1);
                                asIMAP: IMAP.UIDDeleteMsg(IncomingEmails[i].UID);
                            end;
                        end;
                    end;
                end;
                if FormSettings.CheckBoxLogDebug.Checked then begin
                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Parsing incoming e-mails VERCODE finished.');
                end;
                if ValidVersionCodeRequestEmailReceived then begin
                    //* Empty line between
                    LogStrings(FormSettings.EditLogFileName.Text, '');
                end;
            finally
                FreeEmails(IncomingEmails);
                FreeAndNil(IncomingEmails);
                if Assigned(POP3) then begin
                    if POP3.Connected then begin
                        POP3.DisconnectNotifyPeer;
                    end;
                    FreeAndNil(POP3);
                end;
                if Assigned(IMAP) then begin
                    if IMAP.Connected then begin
                        IMAP.DisconnectNotifyPeer;
                    end;
                    FreeAndNil(IMAP);
                end;
                if Assigned(SSL) then begin
                    FreeAndNil(SSL);
                end;
            end;
        end;

        if (IncomingEmailAccount.HostName = '')
        OR (IncomingEmailAccount.LoginName = '')
        OR (IncomingEmailAccount.Password = '')
        then begin
            Exit;
        end;

        if FormSettings.CheckBoxLogDebug.Checked then begin
            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Parsing incoming e-mails ORDERS...');
        end;

        IncomingEmails := TList<TIdMessage>.Create;
        try
            case IncomingEmailAccount.ServerType of
                asPOP3: POP3 := TIDPOP3.Create;
                asIMAP: IMAP := TIdIMAP4.Create;
            else
                Exit;
            end;
            if IncomingEmailAccount.Security = asSSLTLS then begin
                SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
            end;
            case IncomingEmailAccount.ServerType of
                asPOP3: begin
                    if NOT ConnectPOP3(POP3, SSL, IncomingEmailAccount) then begin
                        if FormSettings.CheckBoxLogDebug.Checked then begin
                            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' ConnectPOP3 EXIT');
                        end;
                        Exit;
                    end;
                    if NOT GetEmailsPOP3(POP3, IncomingEmails) then begin
                        if FormSettings.CheckBoxLogDebug.Checked then begin
                            LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' GetEmailsPOP3 EXIT');
                        end;
                        Exit;
                    end;
                end;
                asIMAP: begin
                    if NOT ConnectIMAP(IMAP, SSL, IncomingEmailAccount) then begin
                        Exit;
                    end;
                    if NOT GetEmailsIMAP(IMAP, IncomingEmails) then begin
                        Exit;
                    end;
                end;
            end;

            SMTPSSL := nil;
            IdSMTP := TIdSMTP.Create;
            if OutgoingEmailAccount.Security = asSSLTLS then begin
                SMTPSSL := TIdSSLIOHandlerSocketOpenSSL.Create;
            end;
            try
                //setup SMTP
                IdSMTP.Host := OutgoingEmailAccount.HostName;
                IdSMTP.Port := OutgoingEmailAccount.Port;
                IdSMTP.Username := OutgoingEmailAccount.LoginName;
                IdSMTP.Password := OutgoingEmailAccount.Password;
                {
                IdSMTP.AuthType := satSASL;

                FIdSASLCRAMSHA1 := TIdSASLCRAMSHA1.Create;
                FIdSASLCRAMMD5 := TIdSASLCRAMMD5.Create;
                FIdSASLDigest := TIdSASLDigest.Create;
                FIdSASLSKey := TIdSASLSKey.Create;
                FIdSASLOTP := TIdSASLOTP.Create;
                FIdSASLAnonymous := TIdSASLAnonymous.Create;
                FIdSASLExternal := TIdSASLExternal.Create;
                FIdSASLLogin := TIdSASLLogin.Create;
                FIdSASLPlain := TIdSASLPlain.Create;

                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLCRAMSHA1;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLCRAMMD5;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLDigest;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLSKey;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLOTP;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLAnonymous;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLExternal;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLLogin;
                IdSMTP.SASLMechanisms.Add.SASL := FIdSASLPlain;
                }
                if OutgoingEmailAccount.Security = asSSLTLS then begin
                    SMTPSSL.Host := IdSMTP.Host;
                    SMTPSSL.Port := IdSMTP.Port;
                    SMTPSSL.Destination := SMTPSSL.Host + ':' + IntToStr(SMTPSSL.Port);
                    SMTPSSL.SSLOptions.Method := OutgoingEmailAccount.SSLMethod; //sslvSSLv3; {good} // sslvTLSv1_2 for 3delite.hu
                    IdSMTP.IOHandler := SMTPSSL;
                    IdSMTP.UseTLS := utUseImplicitTLS;
                end;
                try
                    IdSMTP.Connect;
                except on E:Exception do begin
                        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + 'EXCEPTION: ' + E.Message);
                        MessageDlg('ERROR: ' + E.Message, mtWarning, [mbCancel], 0);
                        Exit;
                    end;
                end;

                //* Check to store an order e-mail and generate a registration details e-mail
                for i := 0 to IncomingEmails.Count - 1 do begin
                    ForwardEmail := TIdMessage.Create;
                    try
                        CloneEmail(IncomingEmails[i], ForwardEmail);
                        ForwardEmail.Recipients.EMailAddresses := FormSettings.EditForwardAddress.Text;
                        try
                            IdSMTP.Send(ForwardEmail);
                        except on E:Exception do begin
                                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + 'EXCEPTION: ' + E.Message);
                                Continue;
                            end;
                        end;
                    finally
                        FreeAndNil(ForwardEmail);
                    end;
                    //* Check to add to database end send e-mails
                    SendEmails := TList<TIdMessage>.Create;
                    try
                        if CheckEmailListToAddToDatabase(IncomingEmails[i], SendEmails) then begin
                            ProcessedOrderEmail := True;
                        end;
                        for k := 0 to SendEmails.Count - 1 do begin
                            try
                                //ErrorSendingRegistrationEMail := False;
                                IdSMTP.Send(SendEmails[k]);
                                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' E-mail sent: ''' + SendEmails[k].Recipients[0].Name + ''' <' + SendEmails[k].Recipients[0].Address + '> - ' + SendEmails[k].Subject);
                            except
                                On E: Exception do begin
                                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' EXCEPTION: Send e-mail: ''' + SendEmails[k].Recipients[0].Name + ''' <' + SendEmails[k].Recipients[0].Address + '> - ' + SendEmails[k].Subject + #13#10 + E.Message);
                                end;
                            end;
                        end;
                        case IncomingEmailAccount.ServerType of
                            asPOP3: begin
                                Success := POP3.Delete(i + 1);
                                if NOT Success then begin
                                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + 'ERROR deleting incoming e-mail POP3: ' + IntToStr(i + 1));
                                end;
                            end;
                            asIMAP: begin
                                Success := IMAP.UIDDeleteMsg(IncomingEmails[i].UID);
                                if NOT Success then begin
                                    LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + 'ERROR deleting incoming e-mail IMAP: ' + IncomingEmails[i].UID);
                                end;
                            end;
                        end;
                        if SendEmails.Count > 0 then begin
                            //* Empty line between
                            LogStrings(FormSettings.EditLogFileName.Text, '');
                        end;
                    finally
                        FreeEmails(SendEmails);
                        FreeAndNil(SendEmails);
                    end;
                end;

                if IdSMTP.Connected then begin
                    IdSMTP.Disconnect;
                end;

            finally
                FreeAndNil(IdSMTP);
                if Assigned(SMTPSSL) then begin
                    FreeAndNil(SMTPSSL);
                end;
            end;

            FormTool.ReloadRegistrations;

            //{
            if ProcessedOrderEmail then begin
                if FileExists(DataBaseFileName)
                AND DirectoryExists(DataBaseFileNameBackup)
                then begin
                    FormTool.TimerDataBaseBackup.Enabled := False;
                    FormTool.TimerDataBaseBackup.Enabled := True;
                    if FormSettings.CheckBoxLogDebug.Checked then begin
                        LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' FormTool.TimerDataBaseBackup.Enabled := True');
                    end;
                end;
            end;
            //}

            if FormSettings.CheckBoxLogDebug.Checked then begin
                LogStrings(FormSettings.EditLogFileName.Text, DateTimeToStr(Now) + ' Parsing incoming e-mails ORDERS finished.');
            end;
        finally
            FreeEmails(IncomingEmails);
            FreeAndNil(IncomingEmails);
            if Assigned(POP3) then begin
                if POP3.Connected then begin
                    POP3.DisconnectNotifyPeer;
                end;
                FreeAndNil(POP3);
            end;
            if Assigned(IMAP) then begin
                if IMAP.Connected then begin
                    IMAP.DisconnectNotifyPeer;
                end;
                FreeAndNil(IMAP);
            end;
            if Assigned(SSL) then begin
                FreeAndNil(SSL);
            end;
        end;
    finally
        Screen.Cursor := crDefault;
    end;
end;

procedure FillProjectsList;
var
    i: Integer;
    ProjectItem: TProjectItem;
begin
    ProjectsList.Clear;
    for i := 0 to FormTool.ListViewProjects.Items.Count - 1 do begin
        ProjectItem.Name := FormTool.ListViewProjects.Items[i].Caption;
        ProjectItem.ID := FormTool.ListViewProjects.Items[i].SubItems[0];
        ProjectItem.TableName := FormTool.ListViewProjects.Items[i].SubItems[1];
        ProjectItem.RegSeed := FormTool.ListViewProjects.Items[i].SubItems[2];
        ProjectItem.DownloadURL := FormTool.ListViewProjects.Items[i].SubItems[3];
        ProjectsList.Add(ProjectItem);
    end;
end;


Initialization

    ProjectsList := TList<TProjectItem>.Create;

Finalization

    FreeAndNil(ProjectsList);

end.
