unit SearchWindow;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, UITypes,
  Dialogs, StdCtrls;

type
  TFormSearch = class(TForm)
    ComboBoxSearch: TComboBox;
    GroupBox1: TGroupBox;
    ButtonClose: TButton;
    Label1: TLabel;
    CheckBoxID: TCheckBox;
    CheckBoxUserName: TCheckBox;
    CheckBoxDate: TCheckBox;
    ButtonFind: TButton;
    CheckBoxEmail: TCheckBox;
    CheckBoxComments: TCheckBox;
    procedure ButtonFindClick(Sender: TObject);
    procedure ComboBoxSearchKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ComboBoxSearchChange(Sender: TObject);
  private
    fLastSelIndex: Integer;
    { Private declarations }
  public
    { Public declarations }
    procedure SaveSettings;
    procedure LoadSettings;
  end;

var
  FormSearch: TFormSearch;

implementation

Uses
    Main,
    ComCtrls,
    IniFiles,
    Settings;

{$R *.dfm}

procedure TFormSearch.ButtonFindClick(Sender: TObject);
var
  Item: TListItem;
  i: Integer;
  Search: String;
  SelIndex: Integer;

    procedure HighLite;
    begin
        Item.Selected := True;
        Item.MakeVisible(False);
    end;

begin

    try
        if ComboBoxSearch.Text = ''
            then Exit;

        if ComboBoxSearch.Items.IndexOf(ComboBoxSearch.Text) = -1
            then ComboBoxSearch.Items.Append(ComboBoxSearch.Text);
        Search := ComboBoxSearch.Text;

        if Search = ''
            then Exit;

        if FormTool.ListViewRegs.Items.Count = 0 then begin
            Exit;
        end;

        Item := nil;

        if CheckBoxID.Checked then begin
            //SelIndex := 0;
            SelIndex := fLastSelIndex;
            Item := FormTool.ListViewRegs.FindCaption(SelIndex, Search, True, True, True);
            if Item <> nil then begin
                HighLite;
                Exit;
            end;
        end;

        i := fLastSelIndex;
        repeat
            Item := nil;
            Item := FormTool.ListViewRegs.Items[i];

            if CheckBoxUserName.Checked then begin
                if Pos(Search, Item.SubItems[0]) > 0 then begin
                    HighLite;
                    Exit;
                end;
            end;

            if CheckBoxEmail.Checked then begin
                if Pos(Search, Item.SubItems[1]) > 0  then begin
                    HighLite;
                    Exit;
                end;
            end;

            if CheckBoxDate.Checked then begin
                if Pos(Search, Item.SubItems[2]) > 0  then begin
                    HighLite;
                    Exit;
                end;
            end;

            if CheckBoxComments.Checked then begin
                if Pos(Search, Item.SubItems[3]) > 0  then begin
                    HighLite;
                    Exit;
                end;
            end;

            Inc(i);
        until i >= FormTool.ListViewRegs.Items.Count;

        Item := nil;

    finally

        if Item = nil
            then MessageDlg(FormTool.STR_SEARCH_NOT_FOUND, mtInformation, [mbOk], 0);

    end;
end;

procedure TFormSearch.ComboBoxSearchKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin

    ButtonFind.Enabled := ComboBoxSearch.Text <> '';

    if Key = VK_RETURN
        then ButtonFindClick(Sender);
end;

procedure TFormSearch.FormShow(Sender: TObject);
begin
    LoadSettings;
end;

procedure TFormSearch.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    SaveSettings;
end;

procedure TFormSearch.SaveSettings;
var
  Ini: TIniFile;
begin
    Ini := TIniFile.Create(GetSettingsFileName);
    try
        Ini.WriteInteger('Settings', 'Search Window Top', Top);
        Ini.WriteInteger('Settings', 'Search Window Left', Left);

        ComboBoxSearch.Items.SaveToFile(ExtractFilePath(GetSettingsFileName) + STR_SEARCH_STRINGS);

        Ini.WriteBool('Settings', 'Search by ID', CheckBoxID.Checked);
        Ini.WriteBool('Settings', 'Search by User Name', CheckBoxUserName.Checked);
        Ini.WriteBool('Settings', 'Search by User e-mail', CheckBoxEmail.Checked);
        Ini.WriteBool('Settings', 'Search by Date', CheckBoxDate.Checked);
        Ini.WriteBool('Settings', 'Search by Comment', CheckBoxComments.Checked);

    finally
        Ini.Free;
    end;
end;

procedure TFormSearch.LoadSettings;
var
  Ini: TIniFile;
begin
    Ini := TIniFile.Create(GetSettingsFileName);
    try
        Top := Ini.ReadInteger('Settings', 'Search Window Top', Top);
        Left := Ini.ReadInteger('Settings', 'Search Window Left', Left);

        if FileExists(ExtractFilePath(GetSettingsFileName) + STR_SEARCH_STRINGS)
            then ComboBoxSearch.Items.LoadFromFile(ExtractFilePath(GetSettingsFileName) + STR_SEARCH_STRINGS);

        CheckBoxID.Checked := Ini.ReadBool('Settings', 'Search by ID', True);
        CheckBoxUserName.Checked := Ini.ReadBool('Settings', 'Search by User Name', True);
        CheckBoxEmail.Checked := Ini.ReadBool('Settings', 'Search by User e-mail', True);
        CheckBoxDate.Checked := Ini.ReadBool('Settings', 'Search by Date', True);
        CheckBoxComments.Checked := Ini.ReadBool('Settings', 'Search by Comment', True);
    finally
        Ini.Free;
    end;
end;

procedure TFormSearch.FormCreate(Sender: TObject);
begin
    fLastSelIndex := 0;
    ComboBoxSearch.Text := '';
end;

procedure TFormSearch.ButtonCloseClick(Sender: TObject);
begin
    Close;
end;

procedure TFormSearch.ComboBoxSearchChange(Sender: TObject);
begin
    ButtonFind.Enabled := ComboBoxSearch.Text <> '';
end;

end.
